@@ -13,7 +13,8 @@ print.sarlm <- function(x, ...)
 	invisible(x)
 }
 
-summary.sarlm <- function(object, correlation = FALSE, Nagelkerke=FALSE, ...)
+summary.sarlm <- function(object, correlation = FALSE, Nagelkerke=FALSE,
+ Hausman=FALSE, ...)
 {
 	if (object$type == "error" || ((object$type == "lag" || 
 		object$type == "mixed") && object$ase)) {
@@ -66,7 +67,7 @@ summary.sarlm <- function(object, correlation = FALSE, Nagelkerke=FALSE, ...)
             nk <- NK.sarlm(object)
             if (!is.null(nk)) object$NK <- nk
         }
-        if (object$type == "error" && !is.null(object$Hcov)) {
+        if (Hausman && object$type == "error" && !is.null(object$Hcov)) {
                 object$Haus <- Hausman.sarlm(object)
         }
 	if (object$type == "error") {
@@ -195,7 +196,9 @@ Hausman.sarlm <- function(object, tol=NULL) {
     attr(parameter, "names") <- "df"
     p.value <- 1 - pchisq(abs(statistic), parameter)
     method <- paste("Spatial Hausman test", fmeth)
-    data.name <- deparse(object$formula)
+    data.name <- strwrap(deparse(object$formula), exdent=4)
+    if (length(data.name) > 1) 
+        data.name <- paste(data.name, collapse="\n    ")
     res <- list(statistic = statistic, parameter = parameter, 
         p.value = p.value, method = method, data.name=data.name)
     class(res) <- "htest"
